@@ -26,6 +26,7 @@ use constant {
   MASK_EXPR_END => ~(F_EXPR_END|F_EXPR),
   MASK_SENTENCE_END => ~(F_KEEP_TOKENS|F_SENTENCE_END|F_EXPR|F_EXPR_END|F_SIDEFF),
   MASK_EVAL => ~(F_EVAL),
+  MASK_SIDEFF => ~(F_SIDEFF),
   F_RESCAN => (F_KEEP_TOKENS|F_EVAL|F_STRING_EVAL|F_CONDITIONAL),
 };
 
@@ -33,10 +34,12 @@ our @EXPORT = ((keys %FLAGS), qw/
   is_module_name
   is_version
   convert_string_tokens
+  convert_string_token_list
   MASK_KEEP_TOKENS
   MASK_EXPR_END
   MASK_SENTENCE_END
   MASK_EVAL
+  MASK_SIDEFF
   F_RESCAN
 /);
 
@@ -65,9 +68,10 @@ sub convert_string_tokens {
   my $org_tokens = shift;
   my @tokens;
   my @copied_tokens = @$org_tokens;
+  my $prev = '';
   while(my $copied_token = shift @copied_tokens) {
     my ($token, $desc) = @$copied_token;
-    if ($desc eq '()') {
+    if ($desc and $desc eq '()' and $prev ne 'WORD') {
       unshift @copied_tokens, @$token;
       next;
     }
@@ -79,14 +83,54 @@ sub convert_string_tokens {
     } elsif ($desc eq 'STRING') {
       push @tokens, $token->[0];
     } elsif ($desc eq 'QUOTED_WORD_LIST') {
-      push @tokens, split / /, $token->[0];
+      push @tokens, grep {defined $_ and $_ ne ''} split /\s/, $token->[0];
     } else {
       push @tokens, $copied_token;
     }
+    $prev = $desc;
   }
   \@tokens;
 }
 
+sub convert_string_token_list {
+  my $org_tokens = shift;
+  my @list;
+  my @tokens;
+  my @copied_tokens = @$org_tokens;
+  my $prev = '';
+  while(my $copied_token = shift @copied_tokens) {
+    my ($token, $desc) = @$copied_token;
+    if ($desc and $desc eq '()' and $prev ne 'WORD') {
+      unshift @copied_tokens, @$token;
+      next;
+    }
+
+    if (!$desc) {
+      push @tokens, $copied_token;
+    } elsif ($desc eq 'VERSION_STRING' or $desc eq 'NUMBER') {
+      push @tokens, $token;
+    } elsif ($desc eq 'STRING') {
+      push @tokens, $token->[0];
+    } elsif ($desc eq 'QUOTED_WORD_LIST') {
+      push @list, grep {defined $_ and $_ ne ''} split /\s/, $token->[0];
+    } elsif ($token eq ',' or $token eq '=>') {
+      push @list, @tokens == 1 ? $tokens[0] : \@tokens;
+      @tokens = ();
+      $prev = '';
+    } elsif ($desc eq ';') {
+      last;
+    } else {
+      push @tokens, $copied_token;
+    }
+    $prev = $desc;
+  }
+  if (@tokens) {
+    push @list, @tokens == 1 ? $tokens[0] : \@tokens;
+  }
+
+  \@list;
+}
+
 1;
 
 __END__
